package cytoPrefuse;

import java.util.Comparator;

import prefuse.data.expression.BooleanLiteral;
import prefuse.data.expression.ColumnExpression;
import prefuse.data.expression.ComparisonPredicate;
import prefuse.data.expression.Predicate;
import prefuse.visual.AggregateItem;
import prefuse.visual.DecoratorItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.HoverPredicate;
import prefuse.visual.expression.InGroupPredicate;

public class LabelSorter implements Comparator {

	 private Predicate hoverPredicate = new HoverPredicate();
	 private Predicate highlightPredicate = new ComparisonPredicate(ComparisonPredicate.EQ, new ColumnExpression(VisualItem.HIGHLIGHT), new BooleanLiteral(true));
	 private Predicate selectedPredicate = new InGroupPredicate( CytoPrefuse.SELECTED);
	 private Predicate shortestPathPredicate = new ComparisonPredicate(ComparisonPredicate.EQ, new ColumnExpression(CytoPrefuse.ON_SHORTEST_PATH), new BooleanLiteral(true));
	 private Predicate singleHighlightPredicate = new InGroupPredicate(CytoPrefuse.SINGLE_HIGHLIGHT);
		
	 public int compare(Object arg0, Object arg1) {
	
		DecoratorItem di1 = (DecoratorItem)arg0;		
		DecoratorItem di2 = (DecoratorItem)arg1;

		VisualItem o1 = di1.getDecoratedItem();
		VisualItem o2 = di2.getDecoratedItem();
		
		if (hoverPredicate.getBoolean(o1)){
			return -1;
		}
		if (hoverPredicate.getBoolean(o2)){
			return 1;
		}

		if (o1 instanceof NodeItem) {
			  NodeItem node = (NodeItem)o1;
			  if (node.getBoolean(CytoPrefuse.USER_INTEREST)){
				  return -1;
			  }
		}
		if (o2 instanceof NodeItem) {
			  NodeItem node = (NodeItem)o2;
			  if (node.getBoolean(CytoPrefuse.USER_INTEREST)){
				  return 1;
			  }
		}
	
		if (o1 instanceof NodeItem &&
				o2 instanceof  AggregateItem){		
	  		  NodeItem node = (NodeItem)o1;
			   if (highlightPredicate.getBoolean(node) ||
						hoverPredicate.getBoolean(node)  ||
						selectedPredicate.getBoolean(node)||
						shortestPathPredicate.getBoolean(node)||
						singleHighlightPredicate.getBoolean(node)){
				   return -1;
			   }else {
				   return 1;
			   }
		}
		if (o2 instanceof NodeItem &&
				o1 instanceof  AggregateItem){		
	  		  NodeItem node = (NodeItem)o2;
			   if (highlightPredicate.getBoolean(node) ||
						hoverPredicate.getBoolean(node)  ||
						selectedPredicate.getBoolean(node)||
						shortestPathPredicate.getBoolean(node)||
						singleHighlightPredicate.getBoolean(node)){
				   return 1;
			   }else {
				   return -1;
			   }
		}
		if (o1 instanceof NodeItem &&
					o2 instanceof NodeItem) {
			NodeItem n1 = (NodeItem)di1.getDecoratedItem();
			NodeItem n2 = (NodeItem)di2.getDecoratedItem();
			
			if (n1.getBoolean(CytoPrefuse.ON_SHORTEST_PATH)){
				return -1;
			}
			if (n2.getBoolean(CytoPrefuse.ON_SHORTEST_PATH)){
				return 1;
			}
			
			if (singleHighlightPredicate.getBoolean(n1)){
				return -1;
			}
			if (singleHighlightPredicate.getBoolean(n2)){
				return 1;
			}
			
			if (hoverPredicate.getBoolean(n1)){
				return -1;
			}
			if (hoverPredicate.getBoolean(n2)){
				return 1;
			}
			if (highlightPredicate.getBoolean(n1)){
				return -1;
			}
			if (highlightPredicate.getBoolean(n2)){
				return 1;
			}
			if (selectedPredicate.getBoolean(n1)){
				return -1;
			}
			if (selectedPredicate.getBoolean(n2)){
				return 1;
			}
			
			if (n1.getBoolean(CytoPrefuse.ON_PATHWAY)){
				return -1;
			}
			if (n2.getBoolean(CytoPrefuse.ON_PATHWAY)){
				return 1;
			}
	
	
			
			if (n1.getDegree() < n2.getDegree())
				return 1;
			if (n1.getDegree() == n2.getDegree())
				return 0;
			else
				return -1;
		}
		else{
			
			int group1Size = di1.getInt(CytoPrefuse.GROUP_SIZE);
			int group2Size = di2.getInt(CytoPrefuse.GROUP_SIZE);
						
			if (group1Size < group2Size)
				return 1;
			if (group1Size == group2Size)
				return 0;
			else
				return -1;
		}
	}


}
