package cytoPrefuse;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;

import javax.swing.JOptionPane;
import javax.swing.SwingConstants;

import prefuse.Visualization;
import prefuse.util.GraphicsLib;
import prefuse.util.display.DisplayLib;
import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributes;
import cytoscape.plugin.CytoscapePlugin;
import cytoscape.util.CytoscapeAction;

public class PluginEntry extends CytoscapePlugin implements PropertyChangeListener {

		/**
	 * This class gets attached to the menu item.
	 */
	

	public class CreateCellLayout extends CytoscapeAction {
	
		private static final long serialVersionUID = 1L;
	
		public static final String CREATE_MENU_TEXT ="Create Cerebral view"; 
		/**
		 * The constructor sets the text that should appear on the menu item.
		 */
		public CreateCellLayout() {
			super (CREATE_MENU_TEXT);
		}
	
		/**
		 * This method is called when the user selects the menu item.
		 */
		public void actionPerformed(ActionEvent ae) {
	
			CyNetwork cyNetwork = Cytoscape.getCurrentNetwork();
			if (cyNetwork.getNodeCount() == 0){
				String message = "Choose a network in the network panel before using this menu option.";
				JOptionPane.showMessageDialog(Cytoscape.getDesktop(), message);				    
				return;
			}
			if (Cytoscape.getNodeAttributes().getAttributeNames().length == 0){
				String message = "The nodes in the current network have no attributes.\n  Please import cell localization node attributes before using this menu option.";
				JOptionPane.showMessageDialog(Cytoscape.getDesktop(), message);				    
				return;
			}
			
			CreateViewDialog cvd = new CreateViewDialog();
			CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
			String t=globalAttributes.getStringAttribute(Cytoscape.getCurrentNetwork().getIdentifier(), "cerebralParameters");
			if (t != null && !t.equals("")){
				cvd.loadFromString(t);
			}			
			cvd.showDialog();
			if (cvd.okToRender){
				CytoscapeLayoutMonitor layoutMonitor = new CytoscapeLayoutMonitor(cvd);
			}
		}

		
	
	}
	public class ShowCellLayout extends CytoscapeAction {
	
		private static final String RESTORE_MENU = "Restore previous Cerebral views";
		private static final long serialVersionUID = -1304975942931876614L;
	
		public ShowCellLayout() {
			super(RESTORE_MENU);
		}
	
		public void actionPerformed(ActionEvent arg0) {
			Iterator networksIt = Cytoscape.getNetworkSet().iterator();
			boolean atLeastOneLoaded = false;
			while (networksIt.hasNext()) {
				CyNetwork network = (CyNetwork) networksIt.next();
				Cytoscape.setCurrentNetwork(network.getIdentifier());
				CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
				String t=globalAttributes.getStringAttribute(Cytoscape.getCurrentNetwork().getIdentifier(), "cerebralParameters");
	
				if (t != null && !t.equals("")){
					CreateViewDialog cvd = new CreateViewDialog();
					cvd.loadFromString(t);
					CytoPrefuse cytoPrefuse = new CytoPrefuse(cvd);					
				//	cytoPrefuse.loadNetwork();
					Visualization vis= cytoPrefuse.getVisualization();			
					vis.run("applyVizMapper");
				
					atLeastOneLoaded = true;
				}
				
			}
			if (!atLeastOneLoaded){
				  String message = "This session does not have any saved Cerebral layouts.\nCreate a layout by choosing " + CreateCellLayout.CREATE_MENU_TEXT + " from the menu.";
				  JOptionPane.showMessageDialog(Cytoscape.getDesktop(), message);
			}
		}
		
	}
	private static final String PLUGINS = "Plugins";
	public PluginEntry(){
//		 create a new action to respond to menu activation
		CreateCellLayout action = new CreateCellLayout();
		// set the preferred menu
		action.setPreferredMenu(PLUGINS);
		// and add it to the menus

		ShowCellLayout showCellLayout = new ShowCellLayout();
		showCellLayout.setPreferredMenu(PLUGINS);

		Cytoscape.getDesktop().getCyMenus().addAction(action);
		Cytoscape.getDesktop().getCyMenus().addAction(showCellLayout);
		
		//Add our control panel
		Cytoscape.getDesktop().getCytoPanel(SwingConstants.WEST).add("Cerebral",null, ControlPanel.instance(),"Cerebral");
		
		Cytoscape.getSwingPropertyChangeSupport().addPropertyChangeListener(Cytoscape.SESSION_LOADED, this);
		
	}

	/**
	 * Gives a description of this plugin.
	 */
	public String describe() {
		return "";
	}

	public void propertyChange(PropertyChangeEvent e) {
		Iterator networksIt = Cytoscape.getNetworkSet().iterator();
		while (networksIt.hasNext()) {
			CyNetwork network = (CyNetwork) networksIt.next();
			Cytoscape.setCurrentNetwork(network.getIdentifier());
			CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
			String t=globalAttributes.getStringAttribute(Cytoscape.getCurrentNetwork().getIdentifier(), "cerebralParameters");

			if (t != null && !t.equals("")){
				CreateViewDialog cvd = new CreateViewDialog();
				cvd.loadFromString(t);
				CytoPrefuse cytoPrefuse = new CytoPrefuse(cvd);					
				//cytoPrefuse.loadNetwork();
				Visualization vis= cytoPrefuse.getVisualization();			
				vis.run("applyVizMapper");
				
				 vis.getDisplay(0).setHighQuality(false);
				    
			     Rectangle2D bounds = vis.getBounds(CytoPrefuse.NODES);
		         GraphicsLib.expand(bounds, 50 + (int)(1/cytoPrefuse.getScale()));
		         DisplayLib.fitViewToBounds(cytoPrefuse, bounds, 2000);
		         vis.runAfter("delayHighQuality", 2000);
			}
			
		}
	}




}
