package prefuse.render;

import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Iterator;

import prefuse.Visualization;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.StrokeLib;
import prefuse.visual.VisualItem;
import cytoPrefuse.CytoPrefuse;
import cytoPrefuse.SearchGridLayout;

public class LineDividerRenderer implements Renderer {

	private int margin;
	private final int LINE_THICKNESS = 10;
	private ArrayList allLabels;
	private int lastFontSize;
	private boolean displayDownstreamGenes;
	private String downstreamLabel;
	private int[] layerBounds;
	private Visualization vis;
	private Rectangle2D r = new Rectangle2D.Double();
	private boolean enabled = true;
	public LineDividerRenderer ( Visualization vis, ArrayList allLabels, boolean displayDownstreamGenes, String downstreamLabel){
		lastFontSize = 0;
		margin =0;
		this.allLabels = allLabels;
		this.displayDownstreamGenes = displayDownstreamGenes;
		if (downstreamLabel == null || downstreamLabel.length() == 0){
			this.downstreamLabel = " ";
		}else{
			this.downstreamLabel = downstreamLabel;
		}
		this.vis =vis;
	}
	
	

	public void render(Graphics2D g, VisualItem item) {
		if (enabled){
			if (layerBounds != null){
				double scale = item.getVisualization().getDisplay(0).getScale();		
				int fontSize = (int) (14 * (1/ scale));		
				if (fontSize != lastFontSize){
					recomputeMargin(fontSize);
				}
				
				String label = item.getString(CytoPrefuse.LABEL);
				int layer = item.getInt(CytoPrefuse.LAYER);
				
				vis.getBounds(CytoPrefuse.NODES,r);
				int lineWidth = (int) (margin +r.getWidth());
				int y  =  layerBounds[layer] * SearchGridLayout.SCALE;
				g.setColor(ColorLib.getColor(0.4f,0.4f,0.4f));
				g.setStroke(StrokeLib.getStroke(LINE_THICKNESS, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
				g.drawLine((int) r.getMinX(), y, (int) (r.getMinX() + lineWidth), y);
				Font font =FontLib.getFont("Verdana", fontSize); 
				FontRenderContext frc = Renderer.DEFAULT_GRAPHICS.getFontRenderContext();
				TextLayout textLayout;
				if (label != null && label.length() > 0){
					textLayout = new TextLayout(label, font, frc);
				}else {
					textLayout = new TextLayout(" ", font, frc);
				}
				Rectangle2D textBounds = textLayout.getBounds();
				
			    g.setFont(font);
				g.drawString(label, (int) (r.getMinX() + lineWidth - textBounds.getWidth()), y-LINE_THICKNESS);
				
				if (displayDownstreamGenes){
					if (label.equalsIgnoreCase((String) allLabels.get(allLabels.size()-1))){
						//The last label.. Add the downstream genes below it			
						textLayout = new TextLayout(downstreamLabel, font, frc);
						textBounds = textLayout.getBounds();
						
						g.drawString(downstreamLabel, (int) (r.getMinX() + lineWidth - textBounds.getWidth()), (int) (y + textBounds.getHeight() + 10));
					}
				}
			}
		}
	}

	private void recomputeMargin(int newFontSize) {
		Iterator labelIt = allLabels.iterator();
		Font font =FontLib.getFont("Verdana", newFontSize); 
		FontRenderContext frc = Renderer.DEFAULT_GRAPHICS.getFontRenderContext();
	
		margin = 0;
		while (labelIt.hasNext()) {
			String label = (String)labelIt.next();
			if (label != null && label.length() > 0){
				TextLayout textLayout = new TextLayout(label, font, frc);
				if (textLayout.getBounds().getWidth() > margin){
					margin = (int) textLayout.getBounds().getWidth();
				}
			}
		}
			
		TextLayout textLayout = new TextLayout(downstreamLabel, font, frc);
		if (textLayout.getBounds().getWidth() > margin){
			margin = (int) textLayout.getBounds().getWidth();
		}
		margin +=10;
		lastFontSize = newFontSize;
	}

	public boolean locatePoint(Point2D p, VisualItem item) {
		return false;
	}

	public void setBounds(VisualItem item) {
		if (layerBounds != null){
			int layer = item.getInt(CytoPrefuse.LAYER);
			vis.getBounds(CytoPrefuse.NODES,r);
			int lineWidth = (int) (margin +r.getWidth());
			int y  =  layerBounds[layer] *SearchGridLayout.SCALE;
			item.setBounds(r.getMinX(), y-LINE_THICKNESS/2, lineWidth, LINE_THICKNESS);
		}
	}



	public void setDividers(int[] layerBounds) {
		this.layerBounds = layerBounds;
	}



	public void enable(boolean showSeparators) {
		enabled = showSeparators;
	}

}
